@@ -23,8 +23,8 @@ import davmail.exception.DavMailException;
 import davmail.exchange.ExchangeSessionFactory;
 import davmail.http.DavGatewayHttpClientFacade;
 import davmail.http.DavGatewaySSLProtocolSocketFactory;
-import davmail.http.request.GetRequest;
 import davmail.http.HttpClientAdapter;
+import davmail.http.request.GetRequest;
 import davmail.imap.ImapServer;
 import davmail.ldap.LdapServer;
 import davmail.pop.PopServer;
@@ -48,7 +48,7 @@ public final class DavGateway {
     private DavGateway() {
     }
 
-    private static final ArrayList<AbstractServer> SERVER_LIST = new ArrayList<AbstractServer>();
+    private static final ArrayList<AbstractServer> SERVER_LIST = new ArrayList<>();
 
     /**
      * Start the gateway, listen on specified smtp and pop3 ports
@@ -175,7 +175,7 @@ public final class DavGateway {
             @Override
             public void run() {
                 String releasedVersion = getReleasedVersion();
-                if (currentVersion != null && currentVersion.length() > 0 && releasedVersion != null && currentVersion.compareTo(releasedVersion) < 0) {
+                if (currentVersion.length() > 0 && releasedVersion != null && currentVersion.compareTo(releasedVersion) < 0) {
                     DavGatewayTray.info(new BundleMessage("LOG_NEW_VERSION_AVAILABLE", releasedVersion));
                 }
 
@@ -241,16 +241,13 @@ public final class DavGateway {
     public static String getReleasedVersion() {
         String version = null;
         if (!Settings.getBooleanProperty("davmail.disableUpdateCheck")) {
-            HttpClientAdapter httpClientAdapter = new HttpClientAdapter(HTTP_DAVMAIL_SOURCEFORGE_NET_VERSION_TXT);
-            try {
+            try (HttpClientAdapter httpClientAdapter = new HttpClientAdapter(HTTP_DAVMAIL_SOURCEFORGE_NET_VERSION_TXT)) {
                 GetRequest getRequest = new GetRequest(HTTP_DAVMAIL_SOURCEFORGE_NET_VERSION_TXT);
-                httpClientAdapter.execute(getRequest);
+                getRequest = httpClientAdapter.executeFollowRedirect(getRequest);
                 version = getRequest.getResponseBodyAsString();
                 LOGGER.debug("DavMail released version: " + version);
             } catch (IOException e) {
                 DavGatewayTray.debug(new BundleMessage("LOG_UNABLE_TO_GET_RELEASED_VERSION"));
-            } finally {
-                httpClientAdapter.close();
             }
         }
         return version;
