@@ -26,7 +26,16 @@ import org.apache.log4j.Logger;
 import org.apache.log4j.PatternLayout;
 import org.apache.log4j.RollingFileAppender;
 
-import java.io.*;
+import java.io.BufferedReader;
+import java.io.BufferedWriter;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.io.OutputStreamWriter;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.Iterator;
@@ -56,8 +65,8 @@ public final class Settings {
     private static final Properties SETTINGS = new Properties() {
         @Override
         public synchronized Enumeration<Object> keys() {
-            Enumeration keysEnumeration = super.keys();
-            TreeSet<String> sortedKeySet = new TreeSet<String>();
+            Enumeration<Object> keysEnumeration = super.keys();
+            TreeSet<String> sortedKeySet = new TreeSet<>();
             while (keysEnumeration.hasMoreElements()) {
                 sortedKeySet.add((String) keysEnumeration.nextElement());
             }
@@ -148,7 +157,7 @@ public final class Settings {
      */
     public static void setDefaultSettings() {
         SETTINGS.put("davmail.mode", "EWS");
-        SETTINGS.put("davmail.url", "https://outlook.office365.com/EWS/Exchange.asmx");
+        SETTINGS.put("davmail.url", O365_URL);
         SETTINGS.put("davmail.popPort", "1110");
         SETTINGS.put("davmail.imapPort", "1143");
         SETTINGS.put("davmail.smtpPort", "1025");
@@ -334,20 +343,18 @@ public final class Settings {
             Properties properties = new Properties();
             properties.putAll(SETTINGS);
             // file lines
-            ArrayList<String> lines = new ArrayList<String>();
+            ArrayList<String> lines = new ArrayList<>();
 
-            BufferedWriter writer = null;
-            try  {
+            try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(configFilePath), StandardCharsets.ISO_8859_1))) {
                 readLines(lines, properties);
 
-                writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(configFilePath), "ISO-8859-1"));
                 for (String value : lines) {
                     writer.write(value);
                     writer.newLine();
                 }
 
                 // write remaining lines
-                Enumeration propertyEnumeration = properties.propertyNames();
+                Enumeration<?> propertyEnumeration = properties.propertyNames();
                 while (propertyEnumeration.hasMoreElements()) {
                     String propertyName = (String) propertyEnumeration.nextElement();
                     writer.write(propertyName+"="+ escapeValue(properties.getProperty(propertyName)));
@@ -355,41 +362,24 @@ public final class Settings {
                 }
             } catch (IOException e) {
                 DavGatewayTray.error(new BundleMessage("LOG_UNABLE_TO_STORE_SETTINGS"), e);
-            } finally {
-                if (writer != null) {
-                    try {
-                        writer.close();
-                    } catch (IOException e) {
-                        DavGatewayTray.debug(new BundleMessage("LOG_ERROR_CLOSING_CONFIG_FILE"), e);
-                    }
-                }
             }
         }
         updateLoggingConfig();
     }
 
     private static void readLines(ArrayList<String> lines, Properties properties) {
-        BufferedReader reader = null;
         try {
             File configFile = new File(configFilePath);
             if (configFile.exists()) {
-                reader = new BufferedReader(new InputStreamReader(new FileInputStream(configFile), "ISO-8859-1"));
-
-                String line;
-                while ((line = reader.readLine()) != null) {
-                    lines.add(convertLine(line, properties));
+                try (BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(configFile), StandardCharsets.ISO_8859_1))) {
+                    String line;
+                    while ((line = reader.readLine()) != null) {
+                        lines.add(convertLine(line, properties));
+                    }
                 }
             }
         } catch (IOException e) {
             DavGatewayTray.error(new BundleMessage("LOG_UNABLE_TO_LOAD_SETTINGS"), e);
-        } finally {
-            if (reader != null) {
-                try {
-                    reader.close();
-                } catch (IOException e) {
-                    DavGatewayTray.debug(new BundleMessage("LOG_ERROR_CLOSING_CONFIG_FILE"), e);
-                }
-            }
         }
     }
 
@@ -615,7 +605,7 @@ public final class Settings {
             keyStart = scope + '.';
         }
         Properties result = new Properties();
-        for (Map.Entry entry : SETTINGS.entrySet()) {
+        for (Map.Entry<Object, Object> entry : SETTINGS.entrySet()) {
             String key = (String) entry.getKey();
             if (key.startsWith(keyStart)) {
                 String value = (String) entry.getValue();
@@ -632,12 +622,14 @@ public final class Settings {
      * @param level    logging level
      */
     public static synchronized void setLoggingLevel(String category, Level level) {
-        String prefix = getLoggingPrefix(category);
-        SETTINGS.setProperty(prefix + category, level.toString());
-        if ("rootLogger".equals(category)) {
-            Logger.getRootLogger().setLevel(level);
-        } else {
-            Logger.getLogger(category).setLevel(level);
+        if (level != null) {
+            String prefix = getLoggingPrefix(category);
+            SETTINGS.setProperty(prefix + category, level.toString());
+            if ("rootLogger".equals(category)) {
+                Logger.getRootLogger().setLevel(level);
+            } else {
+                Logger.getLogger(category).setLevel(level);
+            }
         }
     }
 
@@ -676,8 +668,4 @@ public final class Settings {
                 System.getProperty("os.name").toLowerCase().startsWith("freebsd");
     }
 
-    public static boolean isJava8() {
-        return "1.8".equals(System.getProperty("java.specification.version"));
-    }
-
 }
