@@ -26,8 +26,10 @@ import davmail.exception.WebdavNotAvailableException;
 import davmail.http.DavGatewayHttpClientFacade;
 import davmail.http.DavGatewayOTPPrompt;
 import davmail.http.HttpClientAdapter;
+import davmail.http.URIUtil;
 import davmail.http.request.GetRequest;
 import davmail.http.request.PostRequest;
+import davmail.http.request.ResponseWrapper;
 import davmail.util.StringUtil;
 import org.apache.http.HttpStatus;
 import org.apache.http.client.methods.CloseableHttpResponse;
@@ -39,6 +41,7 @@ import org.apache.http.cookie.Cookie;
 import org.apache.http.impl.client.BasicCookieStore;
 import org.apache.http.impl.cookie.BasicClientCookie;
 import org.apache.log4j.Logger;
+import org.htmlcleaner.BaseToken;
 import org.htmlcleaner.CommentNode;
 import org.htmlcleaner.ContentNode;
 import org.htmlcleaner.HtmlCleaner;
@@ -52,18 +55,22 @@ import java.net.ConnectException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.UnknownHostException;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
+/**
+ * New Exchange form authenticator based on HttpClient 4.
+ */
 public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
     protected static final Logger LOGGER = Logger.getLogger("davmail.exchange.ExchangeSession");
 
     /**
      * Various username fields found on custom Exchange authentication forms
      */
-    protected static final Set<String> USER_NAME_FIELDS = new HashSet<String>();
+    protected static final Set<String> USER_NAME_FIELDS = new HashSet<>();
 
     static {
         USER_NAME_FIELDS.add("username");
@@ -78,7 +85,7 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
     /**
      * Various password fields found on custom Exchange authentication forms
      */
-    protected static final Set<String> PASSWORD_FIELDS = new HashSet<String>();
+    protected static final Set<String> PASSWORD_FIELDS = new HashSet<>();
 
     static {
         PASSWORD_FIELDS.add("password");
@@ -93,7 +100,7 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
      * Various OTP (one time password) fields found on custom Exchange authentication forms.
      * Used to open OTP dialog
      */
-    protected static final Set<String> TOKEN_FIELDS = new HashSet<String>();
+    protected static final Set<String> TOKEN_FIELDS = new HashSet<>();
 
     static {
         TOKEN_FIELDS.add("SafeWordPassword");
@@ -128,7 +135,7 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
     /**
      * Logon form user name fields.
      */
-    private final List<String> usernameInputs = new ArrayList<String>();
+    private final List<String> usernameInputs = new ArrayList<>();
     /**
      * Logon form password field, default is password.
      */
@@ -195,17 +202,16 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
             // get webmail root url
             // providing credentials
             // manually follow redirect
-            GetRequest getRequest = new GetRequest(url);
-            CloseableHttpResponse response = httpClientAdapter.executeFollowRedirects(getRequest);
+            GetRequest getRequest = httpClientAdapter.executeFollowRedirect(new GetRequest(url));
 
-            if (!this.isAuthenticated(getRequest, response)) {
+            if (!this.isAuthenticated(getRequest)) {
                 if (isHttpAuthentication) {
                     int status = getRequest.getStatusCode();
 
                     if (status == HttpStatus.SC_UNAUTHORIZED) {
                         throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
                     } else if (status != HttpStatus.SC_OK) {
-                        throw HttpClientAdapter.buildHttpException(getRequest, getRequest.getResponse());
+                        throw HttpClientAdapter.buildHttpResponseException(getRequest, getRequest.getHttpResponse());
                     }
                     // workaround for basic authentication on /exchange and form based authentication at /owa
                     if ("/owa/auth/logon.aspx".equals(getRequest.getURI().getPath())) {
@@ -220,12 +226,7 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
             close();
             LOGGER.error(exc.getMessage());
             throw exc;
-        } catch (ConnectException exc) {
-            close();
-            BundleMessage message = new BundleMessage("EXCEPTION_CONNECT", exc.getClass().getName(), exc.getMessage());
-            LOGGER.error(message);
-            throw new DavMailException("EXCEPTION_DAVMAIL_CONFIGURATION", message);
-        } catch (UnknownHostException exc) {
+        } catch (ConnectException | UnknownHostException exc) {
             close();
             BundleMessage message = new BundleMessage("EXCEPTION_CONNECT", exc.getClass().getName(), exc.getMessage());
             LOGGER.error(message);
@@ -254,10 +255,8 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
         // Create a local context to avoid cookies in main httpClient
         HttpClientContext context = HttpClientContext.create();
         context.setCookieStore(new BasicCookieStore());
-        try {
-            CloseableHttpResponse response = httpClient.execute(httpGet, context);
+        try (CloseableHttpResponse response = httpClient.execute(httpGet, context)) {
             isHttpAuthentication = response.getStatusLine().getStatusCode() == HttpStatus.SC_UNAUTHORIZED;
-            response.close();
         } catch (IOException e) {
             // ignore
         }
@@ -269,9 +268,9 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
      *
      * @return true if session cookies are available
      */
-    protected boolean isAuthenticated(HttpRequestBase getRequest, CloseableHttpResponse response) {
+    protected boolean isAuthenticated(ResponseWrapper getRequest) {
         boolean authenticated = false;
-        if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK
+        if (getRequest.getStatusCode() == HttpStatus.SC_OK
                 && "/ews/services.wsdl".equalsIgnoreCase(getRequest.getURI().getPath())) {
             // direct EWS access returned wsdl
             authenticated = true;
@@ -291,29 +290,26 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
         return authenticated;
     }
 
-    protected void formLogin(HttpClientAdapter httpClient, GetRequest initRequest, String password) throws IOException {
+    protected void formLogin(HttpClientAdapter httpClient, ResponseWrapper initRequest, String password) throws IOException {
         LOGGER.debug("Form based authentication detected");
 
-        PostRequest postRequest = buildLogonMethod(httpClient, initRequest.getURI(), initRequest.getResponseBodyAsString());
+        PostRequest postRequest = buildLogonMethod(httpClient, initRequest);
         if (postRequest == null) {
             LOGGER.debug("Authentication form not found at " + initRequest.getURI() + ", trying default url");
             postRequest = new PostRequest("/owa/auth/owaauth.dll");
         }
-        postRequest = postLogonMethod(httpClient, postRequest, password);
 
-        exchangeUri = postRequest.getURI();
+        exchangeUri = postLogonMethod(httpClient, postRequest, password).getURI();
     }
 
     /**
      * Try to find logon method path from logon form body.
      *
-     * @param httpClient           httpClientAdapter instance
-     * @param uri                  current form uri
-     * @param responseBodyAsString form body
+     * @param httpClient      httpClientAdapter instance
+     * @param responseWrapper init request response wrapper
      * @return logon method
-     * @throws IOException on error
      */
-    protected PostRequest buildLogonMethod(HttpClientAdapter httpClient, URI uri, String responseBodyAsString) throws IOException {
+    protected PostRequest buildLogonMethod(HttpClientAdapter httpClient, ResponseWrapper responseWrapper) {
         PostRequest logonMethod = null;
 
         // create an instance of HtmlCleaner
@@ -323,12 +319,14 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
         usernameInputs.clear();
 
         try {
-            TagNode node = cleaner.clean(new ByteArrayInputStream(responseBodyAsString.getBytes("UTF-8")));
-            List forms = node.getElementListByName("form", true);
+            URI uri = responseWrapper.getURI();
+            String responseBody = responseWrapper.getResponseBodyAsString();
+            TagNode node = cleaner.clean(new ByteArrayInputStream(responseBody.getBytes(StandardCharsets.UTF_8)));
+            List<? extends TagNode> forms = node.getElementListByName("form", true);
             TagNode logonForm = null;
             // select form
             if (forms.size() == 1) {
-                logonForm = (TagNode) forms.get(0);
+                logonForm = forms.get(0);
             } else if (forms.size() > 1) {
                 for (Object form : forms) {
                     if ("logonForm".equals(((TagNode) form).getAttributeByName("name"))) {
@@ -347,14 +345,14 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                 logonMethod = new PostRequest(getAbsoluteUri(uri, logonMethodPath));
 
                 // retrieve lost inputs attached to body
-                List inputList = node.getElementListByName("input", true);
+                List<? extends TagNode> inputList = node.getElementListByName("input", true);
 
                 for (Object input : inputList) {
                     String type = ((TagNode) input).getAttributeByName("type");
                     String name = ((TagNode) input).getAttributeByName("name");
                     String value = ((TagNode) input).getAttributeByName("value");
                     if ("hidden".equalsIgnoreCase(type) && name != null && value != null) {
-                        ((PostRequest) logonMethod).setParameter(name, value);
+                        logonMethod.setParameter(name, value);
                     }
                     // custom login form
                     if (USER_NAME_FIELDS.contains(name)) {
@@ -363,11 +361,10 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                         passwordInput = name;
                     } else if ("addr".equals(name)) {
                         // this is not a logon form but a redirect form
-                        httpClient.executeFollowRedirects(logonMethod);
-                        logonMethod = buildLogonMethod(httpClient, logonMethod.getURI(), logonMethod.getResponseBodyAsString());
+                        logonMethod = buildLogonMethod(httpClient, httpClient.executeFollowRedirect(logonMethod));
                     } else if (TOKEN_FIELDS.contains(name)) {
                         // one time password, ask it to the user
-                        ((PostRequest) logonMethod).setParameter(name, DavGatewayOTPPrompt.getOneTimePassword());
+                        logonMethod.setParameter(name, DavGatewayOTPPrompt.getOneTimePassword());
                     } else if ("otc".equals(name)) {
                         // captcha image, get image and ask user
                         String pinsafeUser = getAliasFromLogin();
@@ -375,35 +372,29 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                             pinsafeUser = username;
                         }
                         HttpGet pinRequest = new HttpGet("/PINsafeISAFilter.dll?username=" + pinsafeUser);
-                        CloseableHttpResponse pinResponse = httpClient.execute(pinRequest);
-                        try {
+                        try (CloseableHttpResponse pinResponse = httpClient.execute(pinRequest)) {
                             int status = pinResponse.getStatusLine().getStatusCode();
                             if (status != HttpStatus.SC_OK) {
-                                throw HttpClientAdapter.buildHttpException(pinRequest, pinResponse);
+                                throw HttpClientAdapter.buildHttpResponseException(pinRequest, pinResponse.getStatusLine());
                             }
                             BufferedImage captchaImage = ImageIO.read(pinResponse.getEntity().getContent());
                             logonMethod.setParameter(name, DavGatewayOTPPrompt.getCaptchaValue(captchaImage));
-
-                        } finally {
-                            pinResponse.close();
                         }
                     }
                 }
             } else {
-                List frameList = node.getElementListByName("frame", true);
+                List<? extends TagNode> frameList = node.getElementListByName("frame", true);
                 if (frameList.size() == 1) {
-                    String src = ((TagNode) frameList.get(0)).getAttributeByName("src");
+                    String src = frameList.get(0).getAttributeByName("src");
                     if (src != null) {
                         LOGGER.debug("Frames detected in form page, try frame content");
-                        GetRequest newInitMethod = new GetRequest(src);
-                        httpClient.executeFollowRedirects(newInitMethod);
-                        logonMethod = buildLogonMethod(httpClient, newInitMethod.getURI(), newInitMethod.getResponseBodyAsString());
+                        logonMethod = buildLogonMethod(httpClient, httpClient.executeFollowRedirect(new GetRequest(src)));
                     }
                 } else {
                     // another failover for script based logon forms (Exchange 2007)
-                    List scriptList = node.getElementListByName("script", true);
+                    List<? extends TagNode> scriptList = node.getElementListByName("script", true);
                     for (Object script : scriptList) {
-                        List contents = ((TagNode) script).getAllChildren();
+                        List<? extends BaseToken> contents = ((TagNode) script).getAllChildren();
                         for (Object content : contents) {
                             if (content instanceof CommentNode) {
                                 String scriptValue = ((CommentNode) content).getCommentedContent();
@@ -415,9 +406,7 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                                 if (sUrl != null && sLgn != null) {
                                     URI src = getScriptBasedFormURL(uri, sLgn + sUrl);
                                     LOGGER.debug("Detected script based logon, redirect to form at " + src);
-                                    GetRequest newInitMethod = new GetRequest(src);
-                                    httpClient.executeFollowRedirects(newInitMethod);
-                                    logonMethod = buildLogonMethod(httpClient, newInitMethod.getURI(), newInitMethod.getResponseBodyAsString());
+                                    logonMethod = buildLogonMethod(httpClient, httpClient.executeFollowRedirect(new GetRequest(src)));
                                 }
 
                             } else if (content instanceof ContentNode) {
@@ -426,46 +415,40 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                                 String location = StringUtil.getToken(scriptValue, "window.location.replace(\"", "\"");
                                 if (location != null) {
                                     LOGGER.debug("Post logon redirect to: " + location);
-                                    GetRequest newInitMethod = new GetRequest(location);
-                                    httpClient.executeFollowRedirects(logonMethod);
-                                    logonMethod = buildLogonMethod(httpClient, newInitMethod.getURI(), newInitMethod.getResponseBodyAsString());
+                                    logonMethod = buildLogonMethod(httpClient, httpClient.executeFollowRedirect(new GetRequest(location)));
                                 }
                             }
                         }
                     }
                 }
             }
-        } catch (IOException e) {
-            LOGGER.error("Error parsing login form at " + uri);
-        } catch (URISyntaxException e) {
-            LOGGER.error("Error parsing login form at " + uri);
+        } catch (IOException | URISyntaxException e) {
+            LOGGER.error("Error parsing login form at " + responseWrapper.getURI());
         }
 
         return logonMethod;
     }
 
 
-    protected PostRequest postLogonMethod(HttpClientAdapter httpClient, PostRequest logonMethod, String password) throws IOException {
+    protected ResponseWrapper postLogonMethod(HttpClientAdapter httpClient, PostRequest logonMethod, String password) throws IOException {
 
         setAuthFormFields(logonMethod, httpClient, password);
 
         // add exchange 2010 PBack cookie in compatibility mode
-        // TODO
-        //httpClientAdapter.addCookie(new BasicClientCookie(httpClientAdapter.getHost(), "PBack", "0", "/", null, false));
         BasicClientCookie pBackCookie = new BasicClientCookie("PBack", "0");
         pBackCookie.setPath("/");
         pBackCookie.setDomain(httpClientAdapter.getHost());
         httpClient.addCookie(pBackCookie);
 
-        CloseableHttpResponse response = httpClient.executeFollowRedirects(logonMethod);
+        ResponseWrapper resultRequest = httpClient.executeFollowRedirect(logonMethod);
 
         // test form based authentication
-        checkFormLoginQueryString(logonMethod);
+        checkFormLoginQueryString(resultRequest);
 
         // workaround for post logon script redirect
-        if (!isAuthenticated(logonMethod, response)) {
+        if (!isAuthenticated(resultRequest)) {
             // try to get new method from script based redirection
-            logonMethod = buildLogonMethod(httpClient, logonMethod.getURI(), logonMethod.getResponseBodyAsString());
+            logonMethod = buildLogonMethod(httpClient, resultRequest);
 
             if (logonMethod != null) {
                 if (otpPreAuthFound && otpPreAuthRetries < MAX_OTP_RETRIES) {
@@ -477,10 +460,11 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                 }
 
                 // if logonMethod is not null, try to follow redirection
-                response = httpClient.executeFollowRedirects(logonMethod);
-                checkFormLoginQueryString(logonMethod);
+                resultRequest = httpClient.executeFollowRedirect(logonMethod);
+
+                checkFormLoginQueryString(resultRequest);
                 // also check cookies
-                if (!isAuthenticated(logonMethod, response)) {
+                if (!isAuthenticated(resultRequest)) {
                     throwAuthenticationFailed();
                 }
             } else {
@@ -490,25 +474,25 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
         }
 
         // check for language selection form
-        if (logonMethod != null && "/owa/languageselection.aspx".equals(logonMethod.getURI().getPath())) {
+        if ("/owa/languageselection.aspx".equals(resultRequest.getURI().getPath())) {
             // need to submit form
-            logonMethod = submitLanguageSelectionForm(logonMethod.getURI(), logonMethod.getResponseBodyAsString());
+            resultRequest = submitLanguageSelectionForm(resultRequest.getURI(), resultRequest.getResponseBodyAsString());
         }
-        return logonMethod;
+        return resultRequest;
     }
 
-    protected PostRequest submitLanguageSelectionForm(URI uri, String responseBodyAsString) throws IOException {
+    protected ResponseWrapper submitLanguageSelectionForm(URI uri, String responseBodyAsString) throws IOException {
         PostRequest postLanguageFormMethod;
         // create an instance of HtmlCleaner
         HtmlCleaner cleaner = new HtmlCleaner();
 
         try {
             TagNode node = cleaner.clean(responseBodyAsString);
-            List forms = node.getElementListByName("form", true);
+            List<? extends TagNode> forms = node.getElementListByName("form", true);
             TagNode languageForm;
             // select form
             if (forms.size() == 1) {
-                languageForm = (TagNode) forms.get(0);
+                languageForm = forms.get(0);
             } else {
                 throw new IOException("Form not found");
             }
@@ -516,18 +500,18 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
 
             postLanguageFormMethod = new PostRequest(getAbsoluteUri(uri, languageMethodPath));
 
-            List inputList = languageForm.getElementListByName("input", true);
+            List<? extends TagNode> inputList = languageForm.getElementListByName("input", true);
             for (Object input : inputList) {
                 String name = ((TagNode) input).getAttributeByName("name");
                 String value = ((TagNode) input).getAttributeByName("value");
                 if (name != null && value != null) {
-                    ((PostRequest) postLanguageFormMethod).setParameter(name, value);
+                    postLanguageFormMethod.setParameter(name, value);
                 }
             }
-            List selectList = languageForm.getElementListByName("select", true);
+            List<? extends TagNode> selectList = languageForm.getElementListByName("select", true);
             for (Object select : selectList) {
                 String name = ((TagNode) select).getAttributeByName("name");
-                List optionList = ((TagNode) select).getElementListByName("option", true);
+                List<? extends TagNode> optionList = ((TagNode) select).getElementListByName("option", true);
                 String value = null;
                 for (Object option : optionList) {
                     if (((TagNode) option).getAttributeByName("selected") != null) {
@@ -536,21 +520,16 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                     }
                 }
                 if (name != null && value != null) {
-                    ((PostRequest) postLanguageFormMethod).setParameter(name, value);
+                    postLanguageFormMethod.setParameter(name, value);
                 }
             }
-        } catch (IOException e) {
-            String errorMessage = "Error parsing language selection form at " + uri;
-            LOGGER.error(errorMessage);
-            throw new IOException(errorMessage);
-        } catch (URISyntaxException e) {
+        } catch (IOException | URISyntaxException e) {
             String errorMessage = "Error parsing language selection form at " + uri;
             LOGGER.error(errorMessage);
             throw new IOException(errorMessage);
         }
 
-        httpClientAdapter.executeFollowRedirects(postLanguageFormMethod);
-        return postLanguageFormMethod;
+        return httpClientAdapter.executeFollowRedirect(postLanguageFormMethod);
     }
 
     protected void setAuthFormFields(HttpRequestBase logonMethod, HttpClientAdapter httpClient, String password) throws IllegalArgumentException {
@@ -607,7 +586,7 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
         URIBuilder uriBuilder = new URIBuilder(uri);
         if (path != null) {
             // reset query string
-            uriBuilder.setQuery(null);
+            uriBuilder.clearParameters();
             if (path.startsWith("/")) {
                 // path is absolute, replace method path
                 uriBuilder.setPath(path);
@@ -627,7 +606,7 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
         return uriBuilder.build();
     }
 
-    protected URI getScriptBasedFormURL(URI uri, String pathQuery) throws URISyntaxException {
+    protected URI getScriptBasedFormURL(URI uri, String pathQuery) throws URISyntaxException, IOException {
         URIBuilder uriBuilder = new URIBuilder(uri);
         int queryIndex = pathQuery.indexOf('?');
         if (queryIndex >= 0) {
@@ -649,15 +628,14 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
                     }
                 }
             }
-            uriBuilder.setQuery(pathQuery.substring(queryIndex + 1));
+            uriBuilder.setCustomQuery(URIUtil.decode(pathQuery.substring(queryIndex + 1)));
         }
         return uriBuilder.build();
     }
 
-    protected void checkFormLoginQueryString(HttpRequestBase logonMethod) throws DavMailAuthenticationException {
+    protected void checkFormLoginQueryString(ResponseWrapper logonMethod) throws DavMailAuthenticationException {
         String queryString = logonMethod.getURI().getRawQuery();
         if (queryString != null && (queryString.contains("reason=2") || queryString.contains("reason=4"))) {
-            logonMethod.releaseConnection();
             throwAuthenticationFailed();
         }
     }
@@ -719,13 +697,22 @@ public class HC4ExchangeFormAuthenticator implements ExchangeAuthenticator {
         return exchangeUri;
     }
 
+    /**
+     * Return authenticated HttpClient 4 HttpClientAdapter
+     *
+     * @return HttpClientAdapter instance
+     */
+    public HttpClientAdapter getHttpClientAdapter() {
+        return httpClientAdapter;
+    }
+
     /**
      * Authenticated httpClientAdapter (with cookies).
      *
      * @return http client
      */
-    public org.apache.commons.httpclient.HttpClient getHttpClientAdapter() throws DavMailException {
-        org.apache.commons.httpclient.HttpClient oldHttpClient = null;
+    public org.apache.commons.httpclient.HttpClient getHttpClient() throws DavMailException {
+        org.apache.commons.httpclient.HttpClient oldHttpClient;
         oldHttpClient = DavGatewayHttpClientFacade.getInstance(url);
         DavGatewayHttpClientFacade.setCredentials(oldHttpClient, username, password);
         DavGatewayHttpClientFacade.createMultiThreadedHttpConnectionManager(oldHttpClient);
