@@ -24,8 +24,8 @@ import davmail.exception.DavMailAuthenticationException;
 import davmail.http.HttpClientAdapter;
 import davmail.http.request.GetRequest;
 import davmail.http.request.PostRequest;
+import davmail.http.request.ResponseWrapper;
 import davmail.http.request.RestRequest;
-import org.apache.http.Header;
 import org.apache.http.HttpStatus;
 import org.apache.http.client.utils.URIBuilder;
 import org.apache.log4j.Logger;
@@ -51,6 +51,36 @@ public class O365Authenticator implements ExchangeAuthenticator {
     private String password;
     private O365Token token;
 
+    public static String buildAuthorizeUrl(String tenantId, String clientId, String redirectUri, String username) throws IOException {
+        URI uri;
+        try {
+            URIBuilder uriBuilder = new URIBuilder()
+                    .setScheme("https")
+                    .setHost("login.microsoftonline.com")
+                    .addParameter("client_id", clientId)
+                    .addParameter("response_type", "code")
+                    .addParameter("redirect_uri", redirectUri)
+                    .addParameter("response_mode", "query")
+                    .addParameter("login_hint", username);
+
+            // force consent
+            //uriBuilder.addParameter("prompt", "consent")
+            // switch to new v2.0 OIDC compliant endpoint https://docs.microsoft.com/en-us/azure/active-directory/develop/azure-ad-endpoint-comparison
+            if (Settings.getBooleanProperty("davmail.enableOidc", false)) {
+                uriBuilder.setPath("/" + tenantId + "/oauth2/v2.0/authorize")
+                        .addParameter("scope", "openid https://outlook.office365.com/EWS.AccessAsUser.All");
+            } else {
+                uriBuilder.setPath("/" + tenantId + "/oauth2/authorize")
+                        .addParameter("resource", RESOURCE);
+            }
+
+            uri = uriBuilder.build();
+        } catch (URISyntaxException e) {
+            throw new IOException(e);
+        }
+        return uri.toString();
+    }
+
     public void setUsername(String username) {
         if (username.contains("|")) {
             this.userid = username.substring(0, username.indexOf("|"));
@@ -89,25 +119,12 @@ public class O365Authenticator implements ExchangeAuthenticator {
                 return;
             }
 
-            URI uri = new URIBuilder()
-                    .setScheme("https")
-                    .setHost("login.microsoftonline.com")
-                    .setPath("/"+tenantId+"/oauth2/authorize")
-                    .addParameter("client_id", clientId)
-                    .addParameter("response_type", "code")
-                    .addParameter("redirect_uri", redirectUri)
-                    .addParameter("response_mode", "query")
-                    .addParameter("resource", RESOURCE)
-                    .addParameter("login_hint", username)
-                    // force consent
-                    //.addParameter("prompt", "consent")
-                    .build();
-            String url = uri.toString();
+            String url = O365Authenticator.buildAuthorizeUrl(tenantId, clientId, redirectUri, username);
 
             httpClientAdapter = new HttpClientAdapter(url, userid, password);
 
-            GetRequest getMethod = new GetRequest(url);
-            String responseBodyAsString = executeRequest(httpClientAdapter, getMethod);
+            GetRequest getRequest = new GetRequest(url);
+            String responseBodyAsString = executeFollowRedirect(httpClientAdapter, getRequest);
             String code;
             if (!responseBodyAsString.contains("Config=")) {
                 // we are no longer on Microsoft, try ADFS
@@ -124,7 +141,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
                 String canary = config.getString("canary");
                 String sessionId = config.getString("sessionId");
 
-                String referer = getMethod.getURI().toString();
+                String referer = getRequest.getURI().toString();
 
                 RestRequest getCredentialMethod = new RestRequest("https://login.microsoftonline.com/" + tenantId + "/GetCredentialType");
                 getCredentialMethod.setRequestHeader("Accept", "application/json");
@@ -147,8 +164,8 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
                 getCredentialMethod.setJsonBody(jsonObject);
 
-                JSONObject credentialType = executeRequestGetConfig(httpClientAdapter, getCredentialMethod);
-                getCredentialMethod.releaseConnection();
+                JSONObject credentialType = httpClientAdapter.executeRestRequest(getCredentialMethod);
+
                 LOGGER.debug("CredentialType=" + credentialType);
 
                 JSONObject credentials = credentialType.getJSONObject("Credentials");
@@ -158,7 +175,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
                     LOGGER.debug("Detected ADFS, redirecting to " + federationRedirectUrl);
                     code = authenticateRedirectADFS(httpClientAdapter, federationRedirectUrl, url);
                 } else {
-                    PostRequest logonMethod = new PostRequest("https://login.microsoftonline.com/"+tenantId+"/login");
+                    PostRequest logonMethod = new PostRequest("https://login.microsoftonline.com/" + tenantId + "/login");
                     logonMethod.setRequestHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
                     logonMethod.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
 
@@ -172,14 +189,14 @@ public class O365Authenticator implements ExchangeAuthenticator {
                     logonMethod.setParameter("loginfmt", username);
                     logonMethod.setParameter("passwd", password);
 
-                    httpClientAdapter.execute(logonMethod);
-                    responseBodyAsString = logonMethod.getResponseBodyAsString();
+                    responseBodyAsString = httpClientAdapter.executePostRequest(logonMethod);
+                    URI location = logonMethod.getRedirectLocation();
+
                     if (responseBodyAsString != null && responseBodyAsString.indexOf("arrUserProofs") > 0) {
-                        logonMethod = handleMfa(httpClientAdapter, logonMethod, username, clientRequestId);
+                        location = handleMfa(httpClientAdapter, logonMethod, username, clientRequestId);
                     }
 
-                    Header locationHeader = logonMethod.getResponseHeader("Location");
-                    if (locationHeader == null || !locationHeader.getValue().startsWith(redirectUri)) {
+                    if (location == null || !location.toString().startsWith(redirectUri)) {
                         // extract response
                         config = extractConfig(logonMethod.getResponseBodyAsString());
                         if (config.optJSONArray("arrScopes") != null || config.optJSONArray("urlPostRedirect") != null) {
@@ -187,7 +204,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
                             LOGGER.debug(url);
                             throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
                         } else if (config.optString("strServiceExceptionMessage") != null) {
-                            LOGGER.debug("O365 returned error: "+config.optString("strServiceExceptionMessage"));
+                            LOGGER.debug("O365 returned error: " + config.optString("strServiceExceptionMessage"));
                             throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
                         } else if ("50126".equals(config.optString("sErrorCode"))) {
                             throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
@@ -195,8 +212,8 @@ public class O365Authenticator implements ExchangeAuthenticator {
                             throw new DavMailAuthenticationException("LOG_MESSAGE", "Authentication failed, unknown error: " + config);
                         }
                     }
-                    String location = locationHeader.getValue();
-                    code = location.substring(location.indexOf("code=") + 5, location.indexOf("&session_state="));
+                    String query = location.toString();
+                    code = query.substring(query.indexOf("code=") + 5, query.indexOf("&session_state="));
                 }
             }
             LOGGER.debug("Authentication Code: " + code);
@@ -210,8 +227,6 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
         } catch (JSONException e) {
             throw new IOException(e + " " + e.getMessage());
-        } catch (URISyntaxException e) {
-            throw new IOException(e + " " + e.getMessage());
         } finally {
             // do not keep login connections open
             if (httpClientAdapter != null) {
@@ -224,14 +239,12 @@ public class O365Authenticator implements ExchangeAuthenticator {
     private String authenticateRedirectADFS(HttpClientAdapter httpClientAdapter, String federationRedirectUrl, String authorizeUrl) throws IOException {
         // get ADFS login form
         GetRequest logonFormMethod = new GetRequest(federationRedirectUrl);
-
-        httpClientAdapter.execute(logonFormMethod);
-        String responseBodyAsString = logonFormMethod.getResponseBodyAsString();
+        String responseBodyAsString = httpClientAdapter.executeGetRequest(logonFormMethod);
         return authenticateADFS(httpClientAdapter, responseBodyAsString, authorizeUrl);
     }
 
     private String authenticateADFS(HttpClientAdapter httpClientAdapter, String responseBodyAsString, String authorizeUrl) throws IOException {
-        String location;
+        URI location;
 
         if (responseBodyAsString.contains("login.microsoftonline.com")) {
             LOGGER.info("Already authenticated through Basic or NTLM");
@@ -244,19 +257,14 @@ public class O365Authenticator implements ExchangeAuthenticator {
             logonMethod.setParameter("Password", password);
             logonMethod.setParameter("AuthMethod", "FormsAuthentication");
 
-            httpClientAdapter.execute(logonMethod);
-
-            if (logonMethod.getStatusCode() != HttpStatus.SC_MOVED_TEMPORARILY || logonMethod.getResponseHeader("Location") == null) {
+            httpClientAdapter.executePostRequest(logonMethod);
+            location = logonMethod.getRedirectLocation();
+            if (location == null) {
                 throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
             }
-            location = logonMethod.getResponseHeader("Location").getValue();
+
             GetRequest redirectMethod = new GetRequest(location);
-            try {
-                httpClientAdapter.execute(redirectMethod);
-                responseBodyAsString = redirectMethod.getResponseBodyAsString();
-            } finally {
-                redirectMethod.releaseConnection();
-            }
+            responseBodyAsString = httpClientAdapter.executeGetRequest(redirectMethod);
         }
 
         if (!responseBodyAsString.contains("login.microsoftonline.com")) {
@@ -278,11 +286,11 @@ public class O365Authenticator implements ExchangeAuthenticator {
         targetMethod.setParameter("wresult", wresult);
         targetMethod.setParameter("wctx", wctx);
 
-        httpClientAdapter.execute(targetMethod);
-        responseBodyAsString = targetMethod.getResponseBodyAsString();
+        responseBodyAsString = httpClientAdapter.executePostRequest(targetMethod);
+        location = targetMethod.getRedirectLocation();
 
         LOGGER.debug(targetMethod.getURI().toString());
-        LOGGER.debug(targetMethod.getStatusLine());
+        LOGGER.debug(targetMethod.getReasonPhrase());
         LOGGER.debug(responseBodyAsString);
 
         if (targetMethod.getStatusCode() == HttpStatus.SC_OK) {
@@ -292,30 +300,34 @@ public class O365Authenticator implements ExchangeAuthenticator {
                 LOGGER.debug(authorizeUrl);
                 throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
             }
-        } else if (targetMethod.getStatusCode() != HttpStatus.SC_MOVED_TEMPORARILY || targetMethod.getResponseHeader("Location") == null) {
+        } else if (targetMethod.getStatusCode() != HttpStatus.SC_MOVED_TEMPORARILY || location == null) {
             throw new IOException("Unknown ADFS authentication failure");
         }
 
-        location = targetMethod.getResponseHeader("Location").getValue();
-        if (location.startsWith("https://device.login.microsoftonline.com")) {
+        if ("device.login.microsoftonline.com".equals(location.getHost())) {
             location = processDeviceLogin(httpClientAdapter, location);
         }
+        String query = location.getQuery();
+        if (query == null) {
+            // failover for null query with non https URI like urn:ietf:wg:oauth:2.0:oob?code=...
+            query = location.getSchemeSpecificPart();
+        }
 
-        if (location.contains("code=") && location.contains("&session_state=")) {
-            String code = location.substring(location.indexOf("code=") + 5, location.indexOf("&session_state="));
+        if (query.contains("code=") && query.contains("&session_state=")) {
+            String code = query.substring(query.indexOf("code=") + 5, query.indexOf("&session_state="));
             LOGGER.debug("Authentication Code: " + code);
             return code;
         }
         throw new IOException("Unknown ADFS authentication failure");
     }
 
-    private String processDeviceLogin(HttpClientAdapter httpClient, String location) throws IOException {
-        String result = location;
+    private URI processDeviceLogin(HttpClientAdapter httpClient, URI location) throws IOException {
+        URI result = location;
         LOGGER.debug("Proceed to device authentication");
         GetRequest deviceLoginMethod = new GetRequest(location);
 
-        httpClient.execute(deviceLoginMethod);
-        String responseBodyAsString = deviceLoginMethod.getResponseBodyAsString();
+        String responseBodyAsString = httpClient.executeGetRequest(deviceLoginMethod);
+
         if (responseBodyAsString.contains("login.microsoftonline.com")) {
             String ctx = extract("name=\"ctx\" value=\"([^\"]+)\"", responseBodyAsString);
             String flowtoken = extract("name=\"flowtoken\" value=\"([^\"]+)\"", responseBodyAsString);
@@ -326,21 +338,18 @@ public class O365Authenticator implements ExchangeAuthenticator {
             processMethod.setParameter("ctx", ctx);
             processMethod.setParameter("flowtoken", flowtoken);
 
-            try {
-                httpClient.execute(processMethod);
-            } finally {
-                processMethod.releaseConnection();
-            }
+            httpClient.executePostRequest(processMethod);
+            result = processMethod.getRedirectLocation();
 
-            if (processMethod.getStatusCode() != HttpStatus.SC_MOVED_TEMPORARILY || processMethod.getResponseHeader("Location") == null) {
+            if (result == null) {
                 throw new DavMailAuthenticationException("EXCEPTION_AUTHENTICATION_FAILED");
             }
-            result = processMethod.getResponseHeader("Location").getValue();
+
         }
         return result;
     }
 
-    private PostRequest handleMfa(HttpClientAdapter httpClientAdapter, PostRequest logonMethod, String username, String clientRequestId) throws JSONException, IOException {
+    private URI handleMfa(HttpClientAdapter httpClientAdapter, PostRequest logonMethod, String username, String clientRequestId) throws JSONException, IOException {
         JSONObject config = extractConfig(logonMethod.getResponseBodyAsString());
         LOGGER.debug("Config=" + config);
 
@@ -388,9 +397,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
         beginAuthJson.put("Method", "BeginAuth");
         beginAuthMethod.setJsonBody(beginAuthJson);
 
-        httpClientAdapter.execute(beginAuthMethod);
-        config = beginAuthMethod.getJsonResponse();
-        beginAuthMethod.releaseConnection();
+        config = httpClientAdapter.executeRestRequest(beginAuthMethod);
         LOGGER.debug(config);
 
         if (!config.getBoolean("Success")) {
@@ -430,9 +437,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
 
             endAuthMethod.setJsonBody(endAuthJson);
 
-            httpClientAdapter.execute(endAuthMethod);
-            config = endAuthMethod.getJsonResponse();
-            endAuthMethod.releaseConnection();
+            config = httpClientAdapter.executeRestRequest(endAuthMethod);
             LOGGER.debug(config);
             String resultValue = config.getString("ResultValue");
             if ("PhoneAppDenied".equals(resultValue) || "PhoneAppNoResponse".equals(resultValue)) {
@@ -453,7 +458,7 @@ public class O365Authenticator implements ExchangeAuthenticator {
         flowToken = config.getString("FlowToken");
 
         // process auth
-        PostRequest processAuthMethod = new PostRequest("https://login.microsoftonline.com/"+tenantId+"/SAS/ProcessAuth");
+        PostRequest processAuthMethod = new PostRequest("https://login.microsoftonline.com/" + tenantId + "/SAS/ProcessAuth");
         processAuthMethod.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
         processAuthMethod.setParameter("type", type);
         processAuthMethod.setParameter("request", context);
@@ -463,28 +468,18 @@ public class O365Authenticator implements ExchangeAuthenticator {
         processAuthMethod.setParameter("flowToken", flowToken);
         processAuthMethod.setParameter("hpgrequestid", hpgrequestid);
 
-        httpClientAdapter.execute(processAuthMethod);
-        return processAuthMethod;
+        httpClientAdapter.executePostRequest(processAuthMethod);
+        return processAuthMethod.getRedirectLocation();
 
     }
 
-    private String executeRequest(HttpClientAdapter httpClientAdapter, GetRequest getRequest) throws IOException {
+    private String executeFollowRedirect(HttpClientAdapter httpClientAdapter, GetRequest getRequest) throws IOException {
         LOGGER.debug(getRequest.getURI());
-        httpClientAdapter.executeFollowRedirects(getRequest);
-        if (getRequest.getURI().getHost().endsWith("okta.com")) {
+        ResponseWrapper responseWrapper = httpClientAdapter.executeFollowRedirect(getRequest);
+        if (responseWrapper.getURI().getHost().endsWith("okta.com")) {
             throw new DavMailAuthenticationException("LOG_MESSAGE", "Okta authentication not supported, please try O365Interactive");
         }
-
-        return getRequest.getResponseBodyAsString();
-    }
-
-    private JSONObject executeRequestGetConfig(HttpClientAdapter httpClientAdapter, RestRequest restRequest) throws IOException {
-        LOGGER.debug(restRequest.getURI());
-        httpClientAdapter.execute(restRequest);
-
-        JSONObject jsonResponse = restRequest.getJsonResponse();
-        LOGGER.debug(jsonResponse);
-        return jsonResponse;
+        return responseWrapper.getResponseBodyAsString();
     }
 
     public JSONObject extractConfig(String content) throws IOException {
