@@ -27,7 +27,6 @@ import davmail.exchange.ews.GetFolderMethod;
 import davmail.exchange.ews.GetUserConfigurationMethod;
 import davmail.http.DavGatewayHttpClientFacade;
 import org.apache.commons.httpclient.HttpClient;
-import org.apache.http.client.utils.URIBuilder;
 import org.apache.log4j.Logger;
 
 import javax.swing.*;
@@ -35,7 +34,6 @@ import java.io.IOException;
 import java.net.Authenticator;
 import java.net.PasswordAuthentication;
 import java.net.URI;
-import java.net.URISyntaxException;
 
 public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
 
@@ -80,6 +78,8 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
     public void authenticate() throws IOException {
         // allow cross domain requests for Okta form support
         System.setProperty("sun.net.http.allowRestrictedHeaders", "true");
+        // enable NTLM for ADFS support
+        System.setProperty("jdk.http.ntlm.transparentAuth", "allHosts");
 
         // common DavMail client id
         final String clientId = Settings.getProperty("davmail.oauth.clientId", "facd6cff-a294-4415-b59f-c5b01937d7bd");
@@ -95,25 +95,7 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
             return;
         }
 
-        URI uri;
-        try {
-            uri = new URIBuilder()
-                    .setScheme("https")
-                    .setHost("login.microsoftonline.com")
-                    .setPath("/"+tenantId+"/oauth2/authorize")
-                    .addParameter("client_id", clientId)
-                    .addParameter("response_type", "code")
-                    .addParameter("redirect_uri", redirectUri)
-                    .addParameter("response_mode", "query")
-                    .addParameter("resource", resource)
-                    .addParameter("login_hint", username)
-                    // force consent
-                    //.addParameter("prompt", "consent")
-                    .build();
-        } catch (URISyntaxException e) {
-            throw new IOException(e);
-        }
-        final String initUrl = uri.toString();
+        final String initUrl = O365Authenticator.buildAuthorizeUrl(tenantId, clientId, redirectUri, username);
 
         // set default authenticator
         Authenticator.setDefault(new Authenticator() {
@@ -136,21 +118,18 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
             }
         });
 
-        SwingUtilities.invokeLater(new Runnable() {
-            @Override
-            public void run() {
-                try {
-                    o365InteractiveAuthenticatorFrame = new O365InteractiveAuthenticatorFrame();
-                    o365InteractiveAuthenticatorFrame.setO365InteractiveAuthenticator(O365InteractiveAuthenticator.this);
-                    o365InteractiveAuthenticatorFrame.authenticate(initUrl, redirectUri);
-                } catch (NoClassDefFoundError e) {
-                    LOGGER.warn("Unable to load JavaFX (OpenJFX), switch to manual mode");
-                    O365ManualAuthenticatorDialog o365ManualAuthenticatorDialog = new O365ManualAuthenticatorDialog(initUrl);
-                    code = o365ManualAuthenticatorDialog.getCode();
-                    isAuthenticated = code != null;
-                    if (!isAuthenticated) {
-                        errorCode = "User did not provide authentication code";
-                    }
+        SwingUtilities.invokeLater(() -> {
+            try {
+                o365InteractiveAuthenticatorFrame = new O365InteractiveAuthenticatorFrame();
+                o365InteractiveAuthenticatorFrame.setO365InteractiveAuthenticator(O365InteractiveAuthenticator.this);
+                o365InteractiveAuthenticatorFrame.authenticate(initUrl, redirectUri);
+            } catch (NoClassDefFoundError e) {
+                LOGGER.warn("Unable to load JavaFX (OpenJFX), switch to manual mode");
+                O365ManualAuthenticatorDialog o365ManualAuthenticatorDialog = new O365ManualAuthenticatorDialog(initUrl);
+                code = o365ManualAuthenticatorDialog.getCode();
+                isAuthenticated = code != null;
+                if (!isAuthenticated) {
+                    errorCode = "User did not provide authentication code";
                 }
             }
         });
@@ -225,8 +204,7 @@ public class O365InteractiveAuthenticator implements ExchangeAuthenticator {
             }
 
         } catch (Exception e) {
-            LOGGER.error(e + " " + e.getMessage());
-            e.printStackTrace();
+            LOGGER.error(e + " " + e.getMessage(), e);
         }
         System.exit(0);
     }
