@@ -27,6 +27,7 @@ import davmail.util.StringEncryptor;
 import org.apache.http.Consts;
 import org.apache.http.NameValuePair;
 import org.apache.http.client.entity.UrlEncodedFormEntity;
+import org.apache.http.client.methods.CloseableHttpResponse;
 import org.apache.http.message.BasicNameValuePair;
 import org.apache.log4j.Logger;
 import org.codehaus.jettison.json.JSONException;
@@ -60,7 +61,7 @@ public class O365Token {
         this.redirectUri = redirectUri;
         this.tokenUrl = "https://login.microsoftonline.com/"+tenantId+"/oauth2/token";
 
-        ArrayList<NameValuePair> parameters = new ArrayList<NameValuePair>();
+        ArrayList<NameValuePair> parameters = new ArrayList<>();
         parameters.add(new BasicNameValuePair("grant_type", "authorization_code"));
         parameters.add(new BasicNameValuePair("code", code));
         parameters.add(new BasicNameValuePair("redirect_uri", redirectUri));
@@ -90,10 +91,25 @@ public class O365Token {
 
             LOGGER.debug("Access token expires " + new Date(expiresOn));
 
-            String decodedBearer = IOUtil.decodeBase64AsString(accessToken.substring(accessToken.indexOf('.') + 1, accessToken.lastIndexOf('.')) + "==");
-            JSONObject tokenBody = new JSONObject(decodedBearer);
-            LOGGER.debug("Token: " + tokenBody);
-            username = tokenBody.getString("unique_name");
+            // get username from id_token
+            String idToken = jsonToken.optString("id_token");
+            if (idToken != null && idToken.contains(".")) {
+                String decodedJwt = IOUtil.decodeBase64AsString(idToken.substring(idToken.indexOf("."),idToken.lastIndexOf(".")));
+                try {
+                    JSONObject tokenBody = new JSONObject(decodedJwt);
+                    LOGGER.debug("Token: " + tokenBody);
+                    username = tokenBody.getString("unique_name");
+                } catch (JSONException e) {
+                    LOGGER.warn("Invalid id_token "+e.getMessage(), e);
+                }
+            }
+            // failover: get username from bearer
+            if (username == null) {
+                String decodedBearer = IOUtil.decodeBase64AsString(accessToken.substring(accessToken.indexOf('.') + 1, accessToken.lastIndexOf('.')) + "==");
+                JSONObject tokenBody = new JSONObject(decodedBearer);
+                LOGGER.debug("Token: " + tokenBody);
+                username = tokenBody.getString("unique_name");
+            }
 
         } catch (JSONException e) {
             throw new IOException("Exception parsing token", e);
@@ -133,7 +149,7 @@ public class O365Token {
     }
 
     public void refreshToken() throws IOException {
-        ArrayList<NameValuePair> parameters = new ArrayList<NameValuePair>();
+        ArrayList<NameValuePair> parameters = new ArrayList<>();
         parameters.add(new BasicNameValuePair("grant_type", "refresh_token"));
         parameters.add(new BasicNameValuePair("refresh_token", refreshToken));
         parameters.add(new BasicNameValuePair("redirect_uri", redirectUri));
@@ -146,21 +162,18 @@ public class O365Token {
     }
 
     private void executeRequest(RestRequest tokenMethod) throws IOException {
-        HttpClientAdapter httpClientAdapter = new HttpClientAdapter(RESOURCE_URL);
-        try {
-            httpClientAdapter.execute(tokenMethod);
-            setJsonToken(tokenMethod.getJsonResponse());
-
-        } finally {
-            // do not keep login connections open
-            httpClientAdapter.close();
+        // do not keep login connections open (no pooling)
+        try (
+                HttpClientAdapter httpClientAdapter = new HttpClientAdapter(RESOURCE_URL);
+                CloseableHttpResponse response = httpClientAdapter.execute(tokenMethod)
+        ) {
+            setJsonToken(tokenMethod.handleResponse(response));
         }
-
     }
 
     static O365Token build(String tenantId, String clientId, String redirectUri, String code, String password) throws IOException {
         O365Token token = new O365Token(tenantId, clientId, redirectUri, code);
-        if (Settings.getBooleanProperty("davmail.oauth.persistToken", false)) {
+        if (Settings.getBooleanProperty("davmail.oauth.persistToken", true)) {
             try {
                 Settings.storeRefreshToken(encryptToken(token.getRefreshToken(), password), token.getUsername());
             } catch (IOException e) {
@@ -171,12 +184,12 @@ public class O365Token {
     }
 
 
-    static O365Token load(String tenantId, String clientId, String redirectUri, String username, String password) throws IOException {
+    static O365Token load(String tenantId, String clientId, String redirectUri, String username, String password) {
         O365Token token = null;
-        if (Settings.getBooleanProperty("davmail.oauth.persistToken", false)) {
+        if (Settings.getBooleanProperty("davmail.oauth.persistToken", true)) {
             String encryptedRefreshToken = Settings.loadRefreshToken(username);
             if (encryptedRefreshToken != null) {
-                String refreshToken = null;
+                String refreshToken;
                 try {
                     refreshToken = decryptToken(encryptedRefreshToken, password);
                     LOGGER.debug("Loaded stored token for " + username);
