@@ -21,7 +21,7 @@ package davmail.exchange.ews;
 /**
  * Item update option.
  */
-@SuppressWarnings({"JavaDoc", "UnusedDeclaration"})
+@SuppressWarnings({"UnusedDeclaration"})
 public final class SendMeetingCancellations extends AttributeOption {
     private SendMeetingCancellations(String value) {
         super("SendMeetingCancellations", value);
