@@ -34,6 +34,7 @@ import org.apache.commons.httpclient.methods.GetMethod;
 import org.apache.commons.httpclient.params.HttpClientParams;
 import org.apache.commons.httpclient.params.HttpMethodParams;
 import org.apache.commons.httpclient.util.IdleConnectionTimeoutThread;
+import org.apache.http.client.HttpResponseException;
 import org.apache.jackrabbit.webdav.DavException;
 import org.apache.jackrabbit.webdav.MultiStatusResponse;
 import org.apache.jackrabbit.webdav.client.methods.CopyMethod;
@@ -56,7 +57,7 @@ import java.util.regex.Pattern;
 public final class DavGatewayHttpClientFacade {
     static final Logger LOGGER = Logger.getLogger("davmail.http.DavGatewayHttpClientFacade");
 
-    public static final String IE_USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.116 Safari/537.36 Edge/15.15063";
+    public static final String IE_USER_AGENT = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; Trident/6.0; Microsoft Outlook 15.0.4420)";
     static final int MAX_REDIRECTS = 10;
     static final Object LOCK = new Object();
     private static boolean needNTLM;
@@ -67,7 +68,7 @@ public final class DavGatewayHttpClientFacade {
 
     private static IdleConnectionTimeoutThread httpConnectionManagerThread;
 
-    private static Set<MultiThreadedHttpConnectionManager> ALL_CONNECTION_MANAGERS = new HashSet<MultiThreadedHttpConnectionManager>();
+    private static Set<MultiThreadedHttpConnectionManager> ALL_CONNECTION_MANAGERS = new HashSet<>();
 
     static {
         // disable Client-initiated TLS renegotiation
@@ -201,11 +202,11 @@ public final class DavGatewayHttpClientFacade {
 
         if (Settings.getBooleanProperty("davmail.enableKerberos", false)) {
             AuthPolicy.registerAuthScheme("Negotiate", SpNegoScheme.class);
-            ArrayList<String> authPrefs = new ArrayList<String>();
+            ArrayList<String> authPrefs = new ArrayList<>();
             authPrefs.add("Negotiate");
             httpClient.getParams().setParameter(AuthPolicy.AUTH_SCHEME_PRIORITY, authPrefs);
         } else if (!needNTLM) {
-            ArrayList<String> authPrefs = new ArrayList<String>();
+            ArrayList<String> authPrefs = new ArrayList<>();
             authPrefs.add(AuthPolicy.DIGEST);
             authPrefs.add(AuthPolicy.BASIC);
             // exclude NTLM authentication scheme
@@ -437,7 +438,7 @@ public final class DavGatewayHttpClientFacade {
             }
             if (locationValue != null) {
                 currentMethod.releaseConnection();
-                throw new HttpException("Maximum redirections reached");
+                throw new HttpServerErrorException("Maximum redirections reached");
             }
         } catch (IOException e) {
             currentMethod.releaseConnection();
@@ -515,7 +516,7 @@ public final class DavGatewayHttpClientFacade {
         int status = executeHttpMethod(httpClient, deleteMethod);
         // do not throw error if already deleted
         if (status != HttpStatus.SC_OK && status != HttpStatus.SC_NOT_FOUND) {
-            throw DavGatewayHttpClientFacade.buildHttpException(deleteMethod);
+            throw DavGatewayHttpClientFacade.buildHttpResponseException(deleteMethod);
         }
         return status;
     }
@@ -534,7 +535,7 @@ public final class DavGatewayHttpClientFacade {
             int status = executeMethodFollowRedirectOnce(httpClient, method);
 
             if (status != HttpStatus.SC_MULTI_STATUS) {
-                throw buildHttpException(method);
+                throw buildHttpResponseException(method);
             }
             responses = method.getResponseBodyAsMultiStatus().getResponses();
 
@@ -582,7 +583,7 @@ public final class DavGatewayHttpClientFacade {
             int status = executeMethodFollowRedirectOnce(httpClient, method);
 
             if (status != HttpStatus.SC_MULTI_STATUS) {
-                throw buildHttpException(method);
+                throw buildHttpResponseException(method);
             }
             responses = method.getResponses();
 
@@ -631,7 +632,7 @@ public final class DavGatewayHttpClientFacade {
         // disable preemptive authentication
         httpClient.getParams().setParameter(HttpClientParams.PREEMPTIVE_AUTHENTICATION, false);
 
-        ArrayList<String> authPrefs = new ArrayList<String>();
+        ArrayList<String> authPrefs = new ArrayList<>();
         authPrefs.add(AuthPolicy.NTLM);
         authPrefs.add(AuthPolicy.DIGEST);
         authPrefs.add(AuthPolicy.BASIC);
@@ -728,7 +729,7 @@ public final class DavGatewayHttpClientFacade {
             if (status != HttpStatus.SC_NOT_FOUND && status != HttpStatus.SC_FORBIDDEN) {
                 LOGGER.warn(method.getResponseBodyAsString());
             }
-            throw DavGatewayHttpClientFacade.buildHttpException(method);
+            throw DavGatewayHttpClientFacade.buildHttpResponseException(method);
         }
         // check for expired session
         if (followRedirects) {
@@ -757,7 +758,7 @@ public final class DavGatewayHttpClientFacade {
      * @param method Http Method
      * @return Http Exception
      */
-    public static HttpException buildHttpException(HttpMethod method) {
+    public static HttpResponseException buildHttpResponseException(HttpMethod method) {
         int status = method.getStatusCode();
         StringBuilder message = new StringBuilder();
         message.append(status).append(' ').append(method.getStatusText());
@@ -781,7 +782,7 @@ public final class DavGatewayHttpClientFacade {
         } else if (status == HttpStatus.SC_INTERNAL_SERVER_ERROR) {
             return new HttpServerErrorException(message.toString());
         } else {
-            return new HttpException(message.toString());
+            return new HttpResponseException(status, message.toString());
         }
     }
 
