@@ -25,15 +25,21 @@ import davmail.exception.HttpNotFoundException;
 import davmail.exception.HttpPreconditionFailedException;
 import davmail.exception.HttpServerErrorException;
 import davmail.exception.LoginTimeoutException;
-import org.apache.commons.httpclient.HttpException;
+import davmail.http.request.ExchangeDavRequest;
+import davmail.http.request.ExchangeSearchRequest;
+import davmail.http.request.GetRequest;
+import davmail.http.request.PostRequest;
+import davmail.http.request.ResponseWrapper;
+import davmail.http.request.RestRequest;
 import org.apache.http.Header;
 import org.apache.http.HttpResponse;
 import org.apache.http.HttpStatus;
+import org.apache.http.StatusLine;
 import org.apache.http.auth.AuthSchemeProvider;
 import org.apache.http.auth.AuthScope;
 import org.apache.http.auth.NTCredentials;
 import org.apache.http.client.CredentialsProvider;
-import org.apache.http.client.ResponseHandler;
+import org.apache.http.client.HttpResponseException;
 import org.apache.http.client.config.AuthSchemes;
 import org.apache.http.client.config.RequestConfig;
 import org.apache.http.client.methods.CloseableHttpResponse;
@@ -55,17 +61,19 @@ import org.apache.http.impl.client.BasicCookieStore;
 import org.apache.http.impl.client.BasicCredentialsProvider;
 import org.apache.http.impl.client.CloseableHttpClient;
 import org.apache.http.impl.client.HttpClientBuilder;
-import org.apache.http.impl.client.IdleConnectionEvictor;
 import org.apache.http.impl.conn.BasicHttpClientConnectionManager;
 import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
 import org.apache.http.impl.conn.SystemDefaultRoutePlanner;
 import org.apache.jackrabbit.webdav.DavException;
 import org.apache.jackrabbit.webdav.MultiStatus;
+import org.apache.jackrabbit.webdav.MultiStatusResponse;
 import org.apache.jackrabbit.webdav.client.methods.BaseDavRequest;
 import org.apache.jackrabbit.webdav.client.methods.HttpCopy;
 import org.apache.jackrabbit.webdav.client.methods.HttpMove;
 import org.apache.log4j.Logger;
+import org.codehaus.jettison.json.JSONObject;
 
+import java.io.Closeable;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
@@ -73,15 +81,14 @@ import java.net.Proxy;
 import java.net.ProxySelector;
 import java.net.URI;
 import java.security.Security;
+import java.util.HashSet;
 import java.util.List;
-import java.util.concurrent.TimeUnit;
 
-public class HttpClientAdapter {
+public class HttpClientAdapter implements Closeable {
     static final Logger LOGGER = Logger.getLogger("davmail.http.HttpClientAdapter");
 
+    static final String[] SUPPORTED_PROTOCOLS = new String[]{"TLSv1", "TLSv1.1", "TLSv1.2"};
     static final Registry<ConnectionSocketFactory> SCHEME_REGISTRY;
-    static final Registry<AuthSchemeProvider> AUTH_SCHEME_REGISTRY;
-    static final RequestConfig DEFAULT_REQUEST_CONFIG;
     static String WORKSTATION_NAME = "UNKNOWN";
     static final int MAX_REDIRECTS = 10;
 
@@ -99,6 +106,7 @@ public class HttpClientAdapter {
         RegistryBuilder<ConnectionSocketFactory> schemeRegistry = RegistryBuilder.create();
         schemeRegistry.register("http", new PlainConnectionSocketFactory());
         schemeRegistry.register("https", new SSLConnectionSocketFactory(new DavGatewaySSLSocketFactory(),
+                SUPPORTED_PROTOCOLS, null,
                 SSLConnectionSocketFactory.getDefaultHostnameVerifier()));
 
         SCHEME_REGISTRY = schemeRegistry.build();
@@ -109,21 +117,6 @@ public class HttpClientAdapter {
             // ignore
         }
 
-        AUTH_SCHEME_REGISTRY = RegistryBuilder.<AuthSchemeProvider>create()
-                .register(AuthSchemes.NTLM, new JCIFSNTLMSchemeFactory())
-                .register(AuthSchemes.BASIC, new BasicSchemeFactory())
-                .register(AuthSchemes.DIGEST, new DigestSchemeFactory())
-                .register(AuthSchemes.SPNEGO, new SPNegoSchemeFactory())
-                .register(AuthSchemes.KERBEROS, new KerberosSchemeFactory())
-                .build();
-
-        DEFAULT_REQUEST_CONFIG = RequestConfig.custom()
-                // socket connect timeout
-                .setConnectTimeout(Settings.getIntProperty("davmail.exchange.connectionTimeout", 10) * 1000)
-                // inactivity timeout
-                .setSocketTimeout(Settings.getIntProperty("davmail.exchange.soTimeout", 120) * 1000)
-                .build();
-
         // set system property *before* calling ProxySelector.getDefault()
         if (Settings.getBooleanProperty("davmail.useSystemProxies", Boolean.FALSE)) {
             System.setProperty("java.net.useSystemProxies", "true");
@@ -131,11 +124,27 @@ public class HttpClientAdapter {
         ProxySelector.setDefault(new DavGatewayProxySelector(ProxySelector.getDefault()));
     }
 
+    /**
+     * Test if the response is gzip encoded
+     *
+     * @param response http response
+     * @return true if response is gzip encoded
+     */
+    public static boolean isGzipEncoded(HttpResponse response) {
+        Header header = response.getFirstHeader("Content-Encoding");
+        return header != null && "gzip".equals(header.getValue());
+    }
+
     HttpClientConnectionManager connectionManager;
     CloseableHttpClient httpClient;
     CredentialsProvider provider = new BasicCredentialsProvider();
-    BasicCookieStore cookieStore = new BasicCookieStore();
-    IdleConnectionEvictor idleConnectionEvictor;
+    BasicCookieStore cookieStore = new BasicCookieStore() {
+        @Override
+        public void addCookie(final Cookie cookie) {
+            LOGGER.debug("Add cookie " + cookie);
+            super.addCookie(cookie);
+        }
+    };
     // current URI
     URI uri;
     String domain;
@@ -154,6 +163,10 @@ public class HttpClientAdapter {
         this(URI.create(url), null, null, enablePool);
     }
 
+    public HttpClientAdapter(String url, String username, String password, boolean enablePool) {
+        this(URI.create(url), username, password, enablePool);
+    }
+
     public HttpClientAdapter(URI uri) {
         this(uri, null, null, false);
     }
@@ -167,16 +180,23 @@ public class HttpClientAdapter {
     }
 
     public HttpClientAdapter(URI uri, String username, String password, boolean enablePool) {
+        // init current uri
+        this.uri = uri;
+
         if (enablePool) {
             connectionManager = new PoolingHttpClientConnectionManager(SCHEME_REGISTRY);
+            ((PoolingHttpClientConnectionManager)connectionManager).setDefaultMaxPerRoute(5);
+            startEvictorThread();
         } else {
             connectionManager = new BasicHttpClientConnectionManager(SCHEME_REGISTRY);
         }
         HttpClientBuilder clientBuilder = HttpClientBuilder.create()
                 .disableRedirectHandling()
-                .setDefaultRequestConfig(DEFAULT_REQUEST_CONFIG)
-                .setUserAgent(DavGatewayHttpClientFacade.IE_USER_AGENT)
-                .setDefaultAuthSchemeRegistry(AUTH_SCHEME_REGISTRY)
+                .setDefaultRequestConfig(getRequestConfig())
+                .setUserAgent(getUserAgent())
+                .setDefaultAuthSchemeRegistry(getAuthSchemeRegistry())
+                // httpClient is not shared between clients, do not track connection state
+                .disableConnectionState()
                 .setConnectionManager(connectionManager);
 
         SystemDefaultRoutePlanner routePlanner = new SystemDefaultRoutePlanner(ProxySelector.getDefault());
@@ -240,6 +260,64 @@ public class HttpClientAdapter {
         httpClient = clientBuilder.build();
     }
 
+    /**
+     * Get current uri host
+     *
+     * @return current host
+     */
+    public String getHost() {
+        return uri.getHost();
+    }
+
+    /**
+     * Force current uri.
+     *
+     * @param uri new uri
+     */
+    public void setUri(URI uri) {
+        this.uri = uri;
+    }
+
+    /**
+     * Current uri.
+     *
+     * @return current uri
+     */
+    public URI getUri() {
+        return uri;
+    }
+
+    private Registry<AuthSchemeProvider> getAuthSchemeRegistry() {
+        final RegistryBuilder<AuthSchemeProvider> registryBuilder = RegistryBuilder.create();
+        registryBuilder.register(AuthSchemes.NTLM, new JCIFSNTLMSchemeFactory())
+                .register(AuthSchemes.BASIC, new BasicSchemeFactory())
+                .register(AuthSchemes.DIGEST, new DigestSchemeFactory());
+        if (Settings.getBooleanProperty("davmail.enableKerberos")) {
+            registryBuilder.register(AuthSchemes.SPNEGO, new SPNegoSchemeFactory())
+                    .register(AuthSchemes.KERBEROS, new KerberosSchemeFactory());
+        }
+
+        return registryBuilder.build();
+    }
+
+    private RequestConfig getRequestConfig() {
+        HashSet<String> authSchemes = new HashSet<>();
+        authSchemes.add(AuthSchemes.NTLM);
+        authSchemes.add(AuthSchemes.BASIC);
+        authSchemes.add(AuthSchemes.DIGEST);
+        if (Settings.getBooleanProperty("davmail.enableKerberos")) {
+            authSchemes.add(AuthSchemes.SPNEGO);
+            authSchemes.add(AuthSchemes.KERBEROS);
+        }
+        return RequestConfig.custom()
+                // socket connect timeout
+                .setConnectTimeout(Settings.getIntProperty("davmail.exchange.connectionTimeout", 10) * 1000)
+                // inactivity timeout
+                .setSocketTimeout(Settings.getIntProperty("davmail.exchange.soTimeout", 120) * 1000)
+                .setTargetPreferredAuthSchemes(authSchemes)
+                .build();
+    }
+
     private void parseUserName(String username) {
         if (username != null) {
             int pipeIndex = username.indexOf("|");
@@ -292,15 +370,12 @@ public class HttpClientAdapter {
     }
 
     public void startEvictorThread() {
-        // TODO: create a common connection evictor for all connection managers
-        idleConnectionEvictor = new IdleConnectionEvictor(connectionManager, 1, TimeUnit.MINUTES);
-        idleConnectionEvictor.start();
+        DavMailIdleConnectionEvictor.addConnectionManager(connectionManager);
     }
 
+    @Override
     public void close() {
-        if (idleConnectionEvictor != null) {
-            idleConnectionEvictor.shutdown();
-        }
+        DavMailIdleConnectionEvictor.removeConnectionManager(connectionManager);
         try {
             httpClient.close();
         } catch (IOException e) {
@@ -336,63 +411,168 @@ public class HttpClientAdapter {
      * @throws IOException on error
      */
     public CloseableHttpResponse execute(HttpRequestBase request, HttpClientContext context) throws IOException {
+        // make sure request path is absolute
+        handleURI(request);
+        // execute request and return response
+        return httpClient.execute(request, context);
+    }
+
+    /**
+     * fix relative uri and update current uri.
+     *
+     * @param request http request
+     */
+    private void handleURI(HttpRequestBase request) {
         URI requestURI = request.getURI();
         if (!requestURI.isAbsolute()) {
             request.setURI(URIUtils.resolve(uri, requestURI));
         }
         uri = request.getURI();
-        CloseableHttpResponse response = httpClient.execute(request, context);
-        if (request instanceof ResponseHandler) {
-            try {
-                ((ResponseHandler) request).handleResponse(response);
-            } finally {
-                response.close();
-            }
+    }
+
+    public ResponseWrapper executeFollowRedirect(PostRequest request) throws IOException {
+        ResponseWrapper responseWrapper = request;
+        LOGGER.debug(request.getMethod()+" "+request.getURI().toString());
+        LOGGER.debug(request.getParameters());
+
+        int count = 0;
+        int maxRedirect = Settings.getIntProperty("davmail.httpMaxRedirects", MAX_REDIRECTS);
+
+        executePostRequest(request);
+        URI redirectLocation = request.getRedirectLocation();
+
+        while (count++ < maxRedirect && redirectLocation != null) {
+            LOGGER.debug("Redirect " + request.getURI() + " to " + redirectLocation);
+            // replace uri with target location
+            responseWrapper = new GetRequest(redirectLocation);
+            executeGetRequest((GetRequest)responseWrapper);
+            redirectLocation = ((GetRequest)responseWrapper).getRedirectLocation();
         }
-        return response;
+
+        return responseWrapper;
+    }
+
+    public GetRequest executeFollowRedirect(GetRequest request) throws IOException {
+        GetRequest result = request;
+        LOGGER.debug(request.getMethod()+" "+request.getURI().toString());
+
+        int count = 0;
+        int maxRedirect = Settings.getIntProperty("davmail.httpMaxRedirects", MAX_REDIRECTS);
+
+        executeGetRequest(request);
+        URI redirectLocation = request.getRedirectLocation();
+
+        while (count++ < maxRedirect && redirectLocation != null) {
+            LOGGER.debug("Redirect " + request.getURI() + " to " + redirectLocation);
+            // replace uri with target location
+            result = new GetRequest(redirectLocation);
+            executeGetRequest(result);
+            redirectLocation = result.getRedirectLocation();
+        }
+
+        return result;
     }
 
     /**
-     * Execute request, manually follow redirects
+     * Execute get request and return response body as string.
      *
-     * @param request Http request
-     * @return Http response
+     * @param getRequest get request
+     * @return response body
      * @throws IOException on error
      */
-    public CloseableHttpResponse executeFollowRedirects(HttpRequestBase request) throws IOException {
-        CloseableHttpResponse httpResponse;
-        int count = 0;
-        int maxRedirect = Settings.getIntProperty("davmail.httpMaxRedirects", MAX_REDIRECTS);
-        httpResponse = execute(request);
-        while (count++ < maxRedirect
-                && isRedirect(httpResponse.getStatusLine().getStatusCode())
-                && httpResponse.getFirstHeader("Location") != null) {
-            // close previous response
-            httpResponse.close();
-            String location = httpResponse.getFirstHeader("Location").getValue();
-            LOGGER.debug("Redirect " + request.getURI() + " to " + location);
-            // replace uri with target location
-            request.setURI(URI.create(location));
-            httpResponse = execute(request);
+    public String executeGetRequest(GetRequest getRequest) throws IOException {
+        handleURI(getRequest);
+        String responseBodyAsString;
+        try (CloseableHttpResponse response = execute(getRequest)) {
+            responseBodyAsString = getRequest.handleResponse(response);
         }
-
-        return httpResponse;
+        return responseBodyAsString;
     }
 
-    public MultiStatus executeDavRequest(BaseDavRequest request) throws IOException, DavException {
+    /**
+     * Execute post request and return response body as string.
+     *
+     * @param postRequest post request
+     * @return response body
+     * @throws IOException on error
+     */
+    public String executePostRequest(PostRequest postRequest) throws IOException {
+        handleURI(postRequest);
+        String responseBodyAsString;
+        try (CloseableHttpResponse response = execute(postRequest)) {
+            responseBodyAsString = postRequest.handleResponse(response);
+        }
+        return responseBodyAsString;
+    }
+
+    public JSONObject executeRestRequest(RestRequest restRequest) throws IOException {
+        handleURI(restRequest);
+        JSONObject responseBody;
+        try (CloseableHttpResponse response = execute(restRequest)) {
+            responseBody = restRequest.handleResponse(response);
+        }
+        return responseBody;
+    }
+
+    /**
+     * Execute WebDav request
+     *
+     * @param request WebDav request
+     * @return multistatus response
+     * @throws IOException on error
+     */
+    public MultiStatus executeDavRequest(BaseDavRequest request) throws IOException {
+        handleURI(request);
         MultiStatus multiStatus = null;
-        CloseableHttpResponse response = execute(request);
-        try {
+        try (CloseableHttpResponse response = execute(request)) {
             request.checkSuccess(response);
             if (response.getStatusLine().getStatusCode() == HttpStatus.SC_MULTI_STATUS) {
                 multiStatus = request.getResponseBodyAsMultiStatus(response);
             }
-        } finally {
-            response.close();
+        } catch (DavException e) {
+            LOGGER.error(e.getMessage(), e);
+            throw new IOException(e.getErrorCode() + " " + e.getStatusPhrase(), e);
         }
         return multiStatus;
     }
 
+    /**
+     * Execute Exchange WebDav request
+     *
+     * @param request WebDav request
+     * @return multistatus response
+     * @throws IOException on error
+     */
+    public MultiStatusResponse[] executeDavRequest(ExchangeDavRequest request) throws IOException {
+        handleURI(request);
+        MultiStatusResponse[] responses;
+        try (CloseableHttpResponse response = execute(request)) {
+            List<MultiStatusResponse> responseList = request.handleResponse(response);
+            // TODO check error handling
+            //request.checkSuccess(response);
+            responses = responseList.toArray(new MultiStatusResponse[0]);
+        }
+        return responses;
+    }
+
+
+    /**
+     * Execute webdav search method.
+     *
+     * @param path            <i>encoded</i> searched folder path
+     * @param searchStatement (SQL like) search statement
+     * @param maxCount        max item count
+     * @return Responses enumeration
+     * @throws IOException on error
+     */
+    public MultiStatusResponse[] executeSearchRequest(String path, String searchStatement, int maxCount) throws IOException {
+        ExchangeSearchRequest searchRequest = new ExchangeSearchRequest(path, searchStatement);
+        if (maxCount > 0) {
+            searchRequest.setHeader("Range", "rows=0-" + (maxCount - 1));
+        }
+        return executeDavRequest(searchRequest);
+    }
+
     public static boolean isRedirect(HttpResponse response) {
         return isRedirect(response.getStatusLine().getStatusCode());
     }
@@ -411,7 +591,7 @@ public class HttpClientAdapter {
     }
 
     /**
-     * Check if status is a redirect (various 30x values).
+     * Get redirect location from header.
      *
      * @param response Http response
      * @return URI target location
@@ -440,16 +620,24 @@ public class HttpClientAdapter {
         cookieStore.addCookie(cookie);
     }
 
+    public String getUserAgent() {
+        return Settings.getProperty("davmail.userAgent", DavGatewayHttpClientFacade.IE_USER_AGENT);
+    }
+
+    public static HttpResponseException buildHttpResponseException(HttpRequestBase request, HttpResponse response) {
+        return buildHttpResponseException(request, response.getStatusLine());
+    }
+
     /**
      * Build Http Exception from method status
      *
      * @param method Http Method
      * @return Http Exception
      */
-    public static HttpException buildHttpException(HttpRequestBase method, HttpResponse response) {
-        int status = response.getStatusLine().getStatusCode();
+    public static HttpResponseException buildHttpResponseException(HttpRequestBase method, StatusLine statusLine) {
+        int status = statusLine.getStatusCode();
         StringBuilder message = new StringBuilder();
-        message.append(status).append(' ').append(response.getStatusLine().getReasonPhrase());
+        message.append(status).append(' ').append(statusLine.getReasonPhrase());
         message.append(" at ").append(method.getURI());
         if (method instanceof HttpCopy || method instanceof HttpMove) {
             message.append(" to ").append(method.getFirstHeader("Destination"));
@@ -457,20 +645,17 @@ public class HttpClientAdapter {
         // 440 means forbidden on Exchange
         if (status == 440) {
             return new LoginTimeoutException(message.toString());
-        } else if (status == org.apache.http.HttpStatus.SC_FORBIDDEN) {
+        } else if (status == HttpStatus.SC_FORBIDDEN) {
             return new HttpForbiddenException(message.toString());
-        } else if (status == org.apache.http.HttpStatus.SC_NOT_FOUND) {
+        } else if (status == HttpStatus.SC_NOT_FOUND) {
             return new HttpNotFoundException(message.toString());
-        } else if (status == org.apache.http.HttpStatus.SC_PRECONDITION_FAILED) {
+        } else if (status == HttpStatus.SC_PRECONDITION_FAILED) {
             return new HttpPreconditionFailedException(message.toString());
-        } else if (status == org.apache.http.HttpStatus.SC_INTERNAL_SERVER_ERROR) {
+        } else if (status == HttpStatus.SC_INTERNAL_SERVER_ERROR) {
             return new HttpServerErrorException(message.toString());
         } else {
-            return new HttpException(message.toString());
+            return new HttpResponseException(status, message.toString());
         }
     }
 
-    public String getHost() {
-        return uri.getHost();
-    }
 }
